/*
 *
 * system monitor plugin
 *
 *
 * @author : b0unc3
 * @ver	   : 1.2
 * @license: BSD
 *
 */

#include <hildon-status-bar-lib/hildon-status-bar-item.h>
#include <gtk/gtk.h>

#include <stdio.h>
#include <sys/statfs.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>

#include <sys/types.h>
#include <net/if.h>
#include <net/if_arp.h>
#include <arpa/inet.h>
#include <sys/ioctl.h>


#define ICON_PLACE	"/usr/share/icons/hicolor/scalable/hildon/"
#define LOW_ICON	ICON_PLACE"hd-25.png"
#define MEDIUM_ICON	ICON_PLACE"hd-50.png"
#define MEDIUM2_ICON	ICON_PLACE"hd-75.png"
#define FULL_ICON	ICON_PLACE"hd-full.png"
#define EMPTY_ICON	ICON_PLACE"hd-empty.png"
#define CPU_ICON	ICON_PLACE"cpu.png"
#define MEM_ICON	ICON_PLACE"mem.png"
#define DISK_ICON	ICON_PLACE"slash.png"
#define MMC_ICON	ICON_PLACE"mmc.png"
#define WRONG_ICON	ICON_PLACE"hd-wrong.png"
#define WIFI_ICON	ICON_PLACE"wifi.png"

#define CPUFILE		"/proc/stat"
#define MEMFILE		"/proc/meminfo"
#define WIFIFILE	"/proc/net/wireless"
#define MOUNTFILE	"/proc/mounts"
#define SWAPFILE	"/proc/swaps"

#define USER 		0
#define NICE		1
#define SYSTEM		2
#define IDLE		3

#define ROOT_PATH	"/"
#define MMC_PATH	"/media/mmc1"

#define RX_FILE 	"/sys/class/net/wlan0/statistics/rx_bytes"
#define TX_FILE		"/sys/class/net/wlan0/statistics/tx_bytes"

#define LIMIT2		((1024*1024))
#define LIMIT		1024

#define MAX_DIGITS_CHARS	32
#define MAX_READ_CHARS		4096


void *system_monitor_initialize(HildonStatusBarItem *item, GtkWidget **button);
void system_monitor_destroy(void *data);
void system_monitor_update(void *data, gint value1,gint value2, const gchar *str);
int system_monitor_get_priority(void *data);

struct _SystemMonitorPlugin
{
	HildonStatusBarItem *item;
	GtkWidget *view;
	GtkWidget *button0;
	GdkPixbuf *pixbuf_25;
	GdkPixbuf *pixbuf_50;
	GdkPixbuf *pixbuf_75;
	GdkPixbuf *pixbuf_curr;
	GdkPixbuf *pixbuf_full;
	GdkPixbuf *pixbuf_empty;
	GdkPixbuf *pixbuf_wrong;
	GdkPixbuf *pixbuf_cpu;
	GdkPixbuf *pixbuf_mem;
	GdkPixbuf *pixbuf_memi;
	GdkPixbuf *pixbuf_disk;
	GdkPixbuf *pixbuf_diski;
	GdkPixbuf *pixbuf_cpui;
	GdkPixbuf *pixbuf_wireless;
	GdkPixbuf *pixbuf_wifi;
	GdkPixbuf *pixbuf_mmc;
	GdkPixbuf *pixbuf_mmci;
	GtkWidget *frame;
	GtkWidget *icon;
	GtkWidget *icon_oth;
	GtkWidget *icon_wi;
	GtkWidget *icon1;
	GtkWidget *icon2;
	guint timeout_times;
	guint timeout_id;
	/*menu*/
	GtkWidget *menu;
	GtkWidget *menu_separator;
	GtkWidget *menu_details;
	GtkWidget *menu_about;
	/* end */
};

typedef struct _SystemMonitorPlugin SystemMonitorPlugin;

static unsigned int times1[4];
static float root_used = 0;
static float mmc_used = 0;
static int free = 0;
static int wifi_link = 0;
static double load = 0;
static long long root_avail = 0;
static long long root_size = 0;
static long long mmc_avail = 0;
static long long mmc_size = 0;
static int m_totavail = 0;
static int m_total = 0;
static gboolean is_mounted = FALSE;


void system_monitor_entry(HildonStatusBarPluginFn_st *fn)
{
	if ( fn == NULL ) {
		return;
	}
	fn->initialize=system_monitor_initialize;
	fn->destroy=system_monitor_destroy;
	fn->update=system_monitor_update;
	fn->get_priority=system_monitor_get_priority;
}

char* get_transf(char *w)
{
	static char leng_str[20];
	unsigned int val = 0;
	char read_buffer[MAX_READ_CHARS];
	char *file_to_open = NULL;
	FILE *f = NULL;

	if ( strcmp(w,"rx") ) {
		file_to_open = RX_FILE;
	} else if ( strcmp(w,"tx") ) {
		file_to_open = TX_FILE;
	} else {
		return NULL;
	}

	if ( (f = fopen(file_to_open,"r")) != NULL ) {
		while ( fgets(read_buffer,MAX_READ_CHARS, f) ) {
			sscanf(read_buffer, "%u", &val);
		}
	}
	fclose(f);
	if ( val > LIMIT2 ) {
		/*int snprintf(char *str, size_t size, const char *format, ...)
		 */
		g_snprintf(leng_str,sizeof(leng_str),"%dM",val/LIMIT2);
	} else if ( val > LIMIT ) {
		g_snprintf(leng_str,sizeof(leng_str),"%dK",val/LIMIT);
	} else {
		g_snprintf(leng_str,sizeof(leng_str),"%d",val);
	}
	return (leng_str);

}

unsigned int *GetTimes(void) {
	static unsigned int times[4];
	char tmp[5];
	char str[80];
	FILE *file;
	
	file = fopen(CPUFILE, "r");
	fgets(str, 80, file);
	sscanf(str, "%s %u %u %u %u", tmp, &times[0], &times[1], &times[2], &times[3]);
	fclose(file);
	return times;
}

static gboolean set_pix(gpointer data)
{
	unsigned int total;
	int i;
	unsigned int *times;
	float percent;
	struct statfs buf;
	FILE *f;
	int memfree = 0;
	int memavail = 0;
	int swapfree = 0;
	int swapavail= 0;
	int memtot = 0;
	char read_buffer[MAX_READ_CHARS];
		 
	SystemMonitorPlugin *smp = (SystemMonitorPlugin *)data;

	if ( statfs(ROOT_PATH, &buf) != 0) {
		g_error("error!!");
	}
	
	root_size = ((long long)buf.f_blocks * (long long)buf.f_bsize ) / 1024 / 1024;
	root_avail = ( (long long)buf.f_bavail * (long long)buf.f_bsize ) / 1024 / 1024;
	percent = ( ( (float)100 / (float)root_size ) * (float)root_avail );
	root_used = 100 - percent;
						

	/*root */
	if ( root_used == 0 ) {
/*		gdk_pixbuf_composite(due,una,0,0,7,23,0,0,1,1,GDK_INTERP_NEAREST, 255);*/
		gdk_pixbuf_composite(smp->pixbuf_empty,smp->pixbuf_diski,0,0,16,40,0,0,1,1,GDK_INTERP_NEAREST, 255);
		gdk_pixbuf_composite(smp->pixbuf_disk,smp->pixbuf_diski,0,20,16,20,0,26,1,1,GDK_INTERP_NEAREST, 255);
		gtk_image_set_from_pixbuf(GTK_IMAGE(smp->icon), smp->pixbuf_diski);
	}
	else if ( root_used > 0 && root_used < 25 ) {
		gdk_pixbuf_composite(smp->pixbuf_25,smp->pixbuf_diski,0,0,16,40,0,0,1,1,GDK_INTERP_NEAREST, 255);
		gdk_pixbuf_composite(smp->pixbuf_disk,smp->pixbuf_diski,0,20,16,20,0,26,1,1,GDK_INTERP_NEAREST, 255);
		gtk_image_set_from_pixbuf(GTK_IMAGE(smp->icon), smp->pixbuf_diski);
	} else if ( root_used > 25 && root_used < 50 ) { 
		gdk_pixbuf_composite(smp->pixbuf_50,smp->pixbuf_diski,0,0,16,40,0,0,1,1,GDK_INTERP_NEAREST, 255);
		gdk_pixbuf_composite(smp->pixbuf_disk,smp->pixbuf_diski,0,20,16,20,0,26,1,1,GDK_INTERP_NEAREST, 255);
		gtk_image_set_from_pixbuf(GTK_IMAGE(smp->icon), smp->pixbuf_diski);
	} else if ( root_used > 50 && root_used < 75 ) {
		gdk_pixbuf_composite(smp->pixbuf_75,smp->pixbuf_diski,0,0,16,40,0,0,1,1,GDK_INTERP_NEAREST, 255);
		gdk_pixbuf_composite(smp->pixbuf_disk,smp->pixbuf_diski,0,20,16,20,0,26,1,1,GDK_INTERP_NEAREST, 255);
		gtk_image_set_from_pixbuf(GTK_IMAGE(smp->icon), smp->pixbuf_diski);
	}
       	else {
		gdk_pixbuf_composite(smp->pixbuf_full,smp->pixbuf_diski,0,0,16,40,0,0,1,1,GDK_INTERP_NEAREST, 255);
		gdk_pixbuf_composite(smp->pixbuf_disk,smp->pixbuf_diski,0,20,16,20,0,26,1,1,GDK_INTERP_NEAREST, 255);
		gtk_image_set_from_pixbuf(GTK_IMAGE(smp->icon), smp->pixbuf_diski);
	}

	/* mount mmc verify */
	f = fopen(MOUNTFILE,"r");
	is_mounted = FALSE;
 	while ( fgets(read_buffer,MAX_READ_CHARS, f) != NULL) {
		if ( strstr(read_buffer,"mmc") != NULL ) {
			is_mounted = TRUE;
		}
	}
	fclose(f);

/*	if ( !is_mounted ) {
		gdk_pixbuf_composite(smp->pixbuf_wrong,smp->pixbuf_mmci, 0,0,16,40,0,0,1,1,GDK_INTERP_NEAREST, 255);
		gdk_pixbuf_composite(smp->pixbuf_mmc,smp->pixbuf_mmci,0,20,16,20,0,26,1,1,GDK_INTERP_NEAREST, 255);
		gtk_image_set_from_pixbuf(GTK_IMAGE(smp->icon_oth), smp->pixbuf_mmci);
		mmc_used = -1;
		mmc_size = -1;
		mmc_avail = -1;
	} else {*/
	if ( statfs(MMC_PATH,&buf) != 0 ) {
		is_mounted=FALSE;
		g_error("error!!");
	}

	if ( !is_mounted ) {
		gdk_pixbuf_composite(smp->pixbuf_wrong,smp->pixbuf_mmci, 0,0,16,40,0,0,1,1,GDK_INTERP_NEAREST, 255);
		gdk_pixbuf_composite(smp->pixbuf_mmc,smp->pixbuf_mmci,0,20,16,20,0,26,1,1,GDK_INTERP_NEAREST, 255);
		gtk_image_set_from_pixbuf(GTK_IMAGE(smp->icon_oth), smp->pixbuf_mmci);
		mmc_used = -1;
		mmc_size = -1;
		mmc_avail = -1;
	} else {

	mmc_size = ((long long)buf.f_blocks * (long long)buf.f_bsize ) / 1024 / 1024;
	mmc_avail = ( (long long)buf.f_bavail * (long long)buf.f_bsize ) / 1024 / 1024;
	percent = ( ( (float)100 / (float)mmc_size ) * (float)mmc_avail );
	mmc_used = 100 - percent;

	/*mmc*/
	if ( mmc_used == 0 ) {
		gdk_pixbuf_composite(smp->pixbuf_empty,smp->pixbuf_mmci,0,0,16,40,0,0,1,1,GDK_INTERP_NEAREST, 255);
		gdk_pixbuf_composite(smp->pixbuf_mmc,smp->pixbuf_mmci,0,20,16,20,0,26,1,1,GDK_INTERP_NEAREST, 255);
		gtk_image_set_from_pixbuf(GTK_IMAGE(smp->icon_oth), smp->pixbuf_mmci);
		
	} else if ( mmc_used > 0 && mmc_used < 45 ) {
		gdk_pixbuf_composite(smp->pixbuf_25,smp->pixbuf_mmci,0,0,16,40,0,0,1,1,GDK_INTERP_NEAREST, 255);
  		gdk_pixbuf_composite(smp->pixbuf_mmc,smp->pixbuf_mmci,0,20,16,20,0,26,1,1,GDK_INTERP_NEAREST, 255);
		gtk_image_set_from_pixbuf(GTK_IMAGE(smp->icon_oth), smp->pixbuf_mmci);
	} else if ( mmc_used > 45 && mmc_used < 65 ) {
		gdk_pixbuf_composite(smp->pixbuf_50,smp->pixbuf_mmci,0,0,16,40,0,0,1,1,GDK_INTERP_NEAREST, 255);
		gdk_pixbuf_composite(smp->pixbuf_mmc,smp->pixbuf_mmci,0,20,16,20,0,26,1,1,GDK_INTERP_NEAREST, 255);
		gtk_image_set_from_pixbuf(GTK_IMAGE(smp->icon_oth), smp->pixbuf_mmci);
	} else if ( mmc_used > 65 && mmc_used < 80 ) {
		gdk_pixbuf_composite(smp->pixbuf_75,smp->pixbuf_mmci,0,0,16,40,0,0,1,1,GDK_INTERP_NEAREST, 255);
		gdk_pixbuf_composite(smp->pixbuf_mmc,smp->pixbuf_mmci,0,20,16,20,0,26,1,1,GDK_INTERP_NEAREST, 255);
		gtk_image_set_from_pixbuf(GTK_IMAGE(smp->icon_oth), smp->pixbuf_mmci);
	}
	else {
		gdk_pixbuf_composite(smp->pixbuf_full,smp->pixbuf_mmci,0,0,16,40,0,0,1,1,GDK_INTERP_NEAREST, 255);
		gdk_pixbuf_composite(smp->pixbuf_mmc,smp->pixbuf_mmci,0,20,16,20,0,26,1,1,GDK_INTERP_NEAREST, 255);
		gtk_image_set_from_pixbuf(GTK_IMAGE(smp->icon_oth), smp->pixbuf_mmci);
	}
	}
	times = GetTimes();
	total=0;
	for(i=0;i<4;i++)
	{
		times1[i] = times[i] - times1[i];
		total += times1[i];
	}
/*	
	total=0;
	for(i=0;i<4;i++)
		total += times1[i];
*/
	load = (double)times1[IDLE] * 100.0 /(double)total;
	load = 100.0 - load;
	/*gdk_pixbuf_composite(smp->pixbuf_25,smp->pixbuf_cpui,0,0,16,40,0,0,1,1,GDK_INTERP_NEAREST, 255);
	gdk_pixbuf_composite(smp->pixbuf_cpu,smp->pixbuf_cpui,0,25,16,15,0,25,1,1,GDK_INTERP_NEAREST, 255);
	gtk_image_set_from_pixbuf(GTK_IMAGE(smp->icon2), smp->pixbuf_cpui);*/
	if ( load == 0 || load < 5 ) {
		gdk_pixbuf_composite(smp->pixbuf_empty,smp->pixbuf_cpui,0,0,16,40,0,0,1,1,GDK_INTERP_NEAREST, 255);
		gdk_pixbuf_composite(smp->pixbuf_cpu,smp->pixbuf_cpui,0,25,16,15,0,25,1,1,GDK_INTERP_NEAREST, 255);
		gtk_image_set_from_pixbuf(GTK_IMAGE(smp->icon2), smp->pixbuf_cpui);
	}
	else if ( load > 5 && load < 30 ) {
		gdk_pixbuf_composite(smp->pixbuf_25,smp->pixbuf_cpui,0,0,16,40,0,0,1,1,GDK_INTERP_NEAREST, 255);
		gdk_pixbuf_composite(smp->pixbuf_cpu,smp->pixbuf_cpui,0,25,16,15,0,25,1,1,GDK_INTERP_NEAREST, 255);
		gtk_image_set_from_pixbuf(GTK_IMAGE(smp->icon2), smp->pixbuf_cpui);
	} else if ( load > 30 && load < 55 ) {
		gdk_pixbuf_composite(smp->pixbuf_50,smp->pixbuf_cpui,0,0,16,40,0,0,1,1,GDK_INTERP_NEAREST, 255);
		gdk_pixbuf_composite(smp->pixbuf_cpu,smp->pixbuf_cpui,0,25,16,15,0,25,1,1,GDK_INTERP_NEAREST, 255);
		gtk_image_set_from_pixbuf(GTK_IMAGE(smp->icon2), smp->pixbuf_cpui);
	} else if ( load > 55 && load < 85 ) {
		gdk_pixbuf_composite(smp->pixbuf_75,smp->pixbuf_cpui,0,0,16,40,0,0,1,1,GDK_INTERP_NEAREST, 255);
		gdk_pixbuf_composite(smp->pixbuf_cpu,smp->pixbuf_cpui,0,25,16,15,0,25,1,1,GDK_INTERP_NEAREST, 255);
		gtk_image_set_from_pixbuf(GTK_IMAGE(smp->icon2), smp->pixbuf_cpui);
	} else if ( load > 85 ) {
		gdk_pixbuf_composite(smp->pixbuf_full,smp->pixbuf_cpui,0,0,16,40,0,0,1,1,GDK_INTERP_NEAREST, 255);
		gdk_pixbuf_composite(smp->pixbuf_cpu,smp->pixbuf_cpui,0,25,16,15,0,25,1,1,GDK_INTERP_NEAREST, 255);
		gtk_image_set_from_pixbuf(GTK_IMAGE(smp->icon2), smp->pixbuf_cpui);
	}
	memcpy(times1, GetTimes(),sizeof(unsigned int) *4);
	
	f = fopen(MEMFILE,"r");
	while ( fgets(read_buffer,MAX_READ_CHARS, f) != NULL) {
		if ( strncmp(read_buffer, "MemFree",7) == 0 ) {
			 sscanf(read_buffer + 13,"%d",&memfree);
		} else if ( strncmp(read_buffer,"SwapFree",8) == 0 ) {
			sscanf(read_buffer + 13 , "%d", &swapfree);
		} else if ( strncmp(read_buffer,"MemTotal",8) == 0 ) {
			sscanf(read_buffer + 13 , "%d" , &memavail);
		} else if ( strncmp(read_buffer,"SwapTotal",9) == 0 ) {
			sscanf(read_buffer + 13 , "%d" , &swapavail);
		}
	}
	fclose(f);

	m_totavail = ( memavail / 1024 ) + ( swapavail / 1024);
	memfree = memfree / 1024;
	swapfree = swapfree / 1024;
	m_total = memfree + swapfree;
	memtot = m_totavail - m_total;
	free = memtot * 100 / m_totavail;

	/*
	gdk_pixbuf_composite(smp->pixbuf_25,smp->pixbuf_memi,0,0,16,40,0,0,1,1,GDK_INTERP_NEAREST, 255);
	gdk_pixbuf_composite(smp->pixbuf_mem,smp->pixbuf_memi,0,25,16,15,0,25,1,1,GDK_INTERP_NEAREST, 255);
	gtk_image_set_from_pixbuf(GTK_IMAGE(smp->icon1), smp->pixbuf_memi);
	*/
	if ( free == 0 ) {
		gdk_pixbuf_composite(smp->pixbuf_empty,smp->pixbuf_memi,0,0,16,40,0,0,1,1,GDK_INTERP_NEAREST, 255);
		gdk_pixbuf_composite(smp->pixbuf_mem,smp->pixbuf_memi,0,25,16,15,0,25,1,1,GDK_INTERP_NEAREST, 255);
		gtk_image_set_from_pixbuf(GTK_IMAGE(smp->icon1), smp->pixbuf_memi);
	}
	else if ( free > 0 && free < 25 ) {
		gdk_pixbuf_composite(smp->pixbuf_25,smp->pixbuf_memi,0,0,16,40,0,0,1,1,GDK_INTERP_NEAREST, 255);
		gdk_pixbuf_composite(smp->pixbuf_mem,smp->pixbuf_memi,0,25,16,15,0,25,1,1,GDK_INTERP_NEAREST, 255);
		gtk_image_set_from_pixbuf(GTK_IMAGE(smp->icon1), smp->pixbuf_memi);
	} else if ( free > 25 && free < 65 ) {
		gdk_pixbuf_composite(smp->pixbuf_50,smp->pixbuf_memi,0,0,16,40,0,0,1,1,GDK_INTERP_NEAREST, 255);
		gdk_pixbuf_composite(smp->pixbuf_mem,smp->pixbuf_memi,0,25,16,15,0,25,1,1,GDK_INTERP_NEAREST, 255);
		gtk_image_set_from_pixbuf(GTK_IMAGE(smp->icon1), smp->pixbuf_memi);
	} else if ( free > 65 && free < 89 ) {
		gdk_pixbuf_composite(smp->pixbuf_75,smp->pixbuf_memi,0,0,16,40,0,0,1,1,GDK_INTERP_NEAREST, 255);
		gdk_pixbuf_composite(smp->pixbuf_mem,smp->pixbuf_memi,0,25,16,15,0,25,1,1,GDK_INTERP_NEAREST, 255);
		gtk_image_set_from_pixbuf(GTK_IMAGE(smp->icon1), smp->pixbuf_memi);
	} else if ( free > 89 ) {
		gdk_pixbuf_composite(smp->pixbuf_full,smp->pixbuf_memi,0,0,16,40,0,0,1,1,GDK_INTERP_NEAREST, 255);
		gdk_pixbuf_composite(smp->pixbuf_mem,smp->pixbuf_memi,0,25,16,15,0,25,1,1,GDK_INTERP_NEAREST, 255);
		gtk_image_set_from_pixbuf(GTK_IMAGE(smp->icon1), smp->pixbuf_memi);
	}

	/* wifi link */
	wifi_link = -1;
	f = fopen(WIFIFILE,"r");
	while ( fgets(read_buffer,MAX_READ_CHARS,f) != NULL) {
		if ( strstr(read_buffer,"wlan0:") != NULL ) {
			sscanf(read_buffer + 13,"%d",&wifi_link);
		}
	}
	fclose(f);

	if ( wifi_link == -1 || wifi_link == 0 ) {
		gdk_pixbuf_composite(smp->pixbuf_wrong,smp->pixbuf_wifi, 0,0,16,40,0,0,1,1,GDK_INTERP_NEAREST, 255);
		gdk_pixbuf_composite(smp->pixbuf_wireless,smp->pixbuf_wifi,0,25,16,15,0,25,1,1,GDK_INTERP_NEAREST, 255);
		gtk_image_set_from_pixbuf(GTK_IMAGE(smp->icon_wi), smp->pixbuf_wifi);
	} else {
	if ( wifi_link > 0 && wifi_link < 25 ) {
		gdk_pixbuf_composite(smp->pixbuf_25,smp->pixbuf_wifi, 0,0,16,40,0,0,1,1,GDK_INTERP_NEAREST, 255);
		gdk_pixbuf_composite(smp->pixbuf_wireless,smp->pixbuf_wifi,0,25,16,15,0,25,1,1,GDK_INTERP_NEAREST, 255);
		gtk_image_set_from_pixbuf(GTK_IMAGE(smp->icon_wi), smp->pixbuf_wifi);
	} else if ( wifi_link > 25 && wifi_link < 55 ) {
		gdk_pixbuf_composite(smp->pixbuf_50,smp->pixbuf_wifi, 0,0,16,40,0,0,1,1,GDK_INTERP_NEAREST, 255);
		gdk_pixbuf_composite(smp->pixbuf_wireless,smp->pixbuf_wifi,0,25,16,15,0,25,1,1,GDK_INTERP_NEAREST, 255);
		gtk_image_set_from_pixbuf(GTK_IMAGE(smp->icon_wi), smp->pixbuf_wifi);
	} else if ( wifi_link > 55 && wifi_link < 85 ) {
		gdk_pixbuf_composite(smp->pixbuf_75,smp->pixbuf_wifi, 0,0,16,40,0,0,1,1,GDK_INTERP_NEAREST, 255);
		gdk_pixbuf_composite(smp->pixbuf_wireless,smp->pixbuf_wifi,0,25,16,15,0,25,1,1,GDK_INTERP_NEAREST, 255);
		gtk_image_set_from_pixbuf(GTK_IMAGE(smp->icon_wi), smp->pixbuf_wifi);
	} else if ( wifi_link > 85 ) {
		gdk_pixbuf_composite(smp->pixbuf_full,smp->pixbuf_wifi, 0,0,16,40,0,0,1,1,GDK_INTERP_NEAREST, 255);
		gdk_pixbuf_composite(smp->pixbuf_wireless,smp->pixbuf_wifi,0,25,16,15,0,25,1,1,GDK_INTERP_NEAREST, 255);
		gtk_image_set_from_pixbuf(GTK_IMAGE(smp->icon_wi), smp->pixbuf_wifi);
	}
	}

	return TRUE;

}
/* menu */

void pos_menu(GtkMenu *menu, gint *x, gint *y, gboolean *in, gpointer data) 
{
	GtkAllocation *a, *ma;
	GtkWidget *toplevel;
	gint tx, ty;

	g_return_if_fail(x && y);
	g_return_if_fail(data);
	g_return_if_fail(GTK_IS_WIDGET(((SystemMonitorPlugin*)(data))->item));

	a = &(GTK_WIDGET(((SystemMonitorPlugin*)(data))->item)->allocation);

	gtk_widget_realize(GTK_WIDGET(menu));
	ma = &GTK_WIDGET(menu)->allocation;

	toplevel =  gtk_widget_get_toplevel(GTK_WIDGET(((SystemMonitorPlugin*)(data))->item));

	if(GTK_WIDGET_TOPLEVEL(toplevel)){
		gdk_window_get_position(toplevel->window, &tx, &ty);
		*x = (tx + a->x + a->width) - ma->width;
	}
	*y = 60;
}


void show_details(GtkMenuItem *item, gpointer data)
{
	GtkWidget *dialog;
	GtkWidget *progressbar;
	GtkWidget *button_ok;
	GtkWidget *notebook;
	GtkWidget *frame;
	GtkWidget *label;
	GtkWidget *hbox;
	GtkWidget *vbox;
	GtkWidget *mvbox;
	GtkWidget *label0;
	GtkWidget *label1;
	GtkWidget *main_frame;
	GtkWidget *wi_frame;
	float r_used = 0;
	float m_used = 0;
	float r_free = 0;
	float wlink;
	char x[64];
	struct ifreq *ifr;
	struct ifreq ifrr;
	struct in_addr addr;
	char *ifname = "wlan0";
	int sock = socket(AF_INET,SOCK_DGRAM,0);
	int count=0;
	FILE *f;

	dialog = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW(dialog), "SMP - Details");
	button_ok = gtk_dialog_add_button(GTK_DIALOG(dialog), "Ok", 0);
	notebook = gtk_notebook_new();
	main_frame = gtk_frame_new("DISK STATUS");
	wi_frame = gtk_frame_new("WIRELESS STATUS");
	vbox = gtk_vbox_new(TRUE,5);
	mvbox = gtk_vbox_new(TRUE,5);
	/* root detail */
	r_used = root_used / 100.0;
	label = gtk_label_new("ROOT (/)");
	sprintf(x,"%LdM - %LdM free",root_size,root_avail);
	label0=gtk_label_new(x);
	progressbar = gtk_progress_bar_new();
	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progressbar),r_used);
	sprintf(x,"%.0f%%%",root_used);
	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progressbar),x);
	hbox = gtk_hbox_new(TRUE,2);

	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE , 0);
	gtk_box_pack_start(GTK_BOX(hbox), progressbar , FALSE, FALSE , 0);
	gtk_box_pack_start(GTK_BOX(hbox), label0 , FALSE, FALSE , 0);
	/* mmc detail */
	gtk_box_pack_start(GTK_BOX(vbox), hbox , FALSE, FALSE , 0);
	gtk_container_add(GTK_CONTAINER(vbox),hbox);
	gtk_container_add(GTK_CONTAINER(main_frame),vbox);
	sprintf(x,"%LdM - %LdM free",mmc_size,mmc_avail);
	if ( !is_mounted ) {
		label0=gtk_label_new("not mounted.");
		m_used=0;
		mmc_used=0;
	} else {
		label0=gtk_label_new(x);
		m_used = mmc_used / 100.0;
	}
	label = gtk_label_new("MMC (/media/mmc1)");
	progressbar = gtk_progress_bar_new();
	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progressbar), m_used);
	sprintf(x,"%.0f%%%",mmc_used);
	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progressbar),x);
	hbox = gtk_hbox_new(TRUE,2);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE , 0);
	gtk_box_pack_start(GTK_BOX(hbox), progressbar , FALSE, FALSE , 0);
	gtk_box_pack_start(GTK_BOX(hbox), label0 , FALSE, FALSE , 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox , FALSE, FALSE , 0);
	gtk_container_add(GTK_CONTAINER(vbox),hbox);
	gtk_container_add(GTK_CONTAINER(main_frame),vbox);
	/* ram detail */
	r_free = free / 100.0;
	if ( (f = fopen(SWAPFILE,"r")) != NULL ) {
		if (fgets(x,64,f) != NULL ) {
			/* swap is on */
			label = gtk_label_new("RAM (+SWAP)");
		} else {
			label = gtk_label_new("RAM");
		}
	}
	sprintf(x,"%dM - %dM free",m_totavail,m_total);
	label0=gtk_label_new(x);
	progressbar = gtk_progress_bar_new();
	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progressbar), r_free);
	sprintf(x,"%d",free);
	strcat(x,"%");
	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progressbar),x);
	hbox = gtk_hbox_new(TRUE,2);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE , 0);
	gtk_box_pack_start(GTK_BOX(hbox), progressbar , FALSE, FALSE , 0);
	gtk_box_pack_start(GTK_BOX(hbox), label0 , FALSE, FALSE , 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox , FALSE, FALSE , 0);
	gtk_container_add(GTK_CONTAINER(main_frame),vbox);
	label1 = gtk_label_new("DISK STAT");
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), main_frame, label1);
	/* wifi detail */
	wlink = wifi_link / 100.0;
	label = gtk_label_new("LINK QUALITY");
	ifr = &ifrr;
	ifrr.ifr_addr.sa_family = AF_INET;
	strncpy(ifrr.ifr_name,ifname,sizeof(ifrr.ifr_name));

	if ( ioctl(sock,SIOCGIFADDR, ifr) < 0 ) {
		close(sock);
		label0=gtk_label_new("not connected.");
	} else {
		memcpy(&addr, ifrr.ifr_addr.sa_data+2,sizeof(addr));
		close(sock);
		label0=gtk_label_new(inet_ntoa(addr));
	}
	progressbar = gtk_progress_bar_new();
	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progressbar), wlink);
	sprintf(x,"%d",wifi_link);
	strcat(x,"%");
	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progressbar),x);
	hbox = gtk_hbox_new(TRUE,2);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE , 0);
	gtk_box_pack_start(GTK_BOX(hbox), progressbar , FALSE, FALSE , 0);
	gtk_box_pack_start(GTK_BOX(mvbox), hbox , FALSE, FALSE , 0);
	hbox = gtk_hbox_new(TRUE,2);
	label = gtk_label_new("IP ADDRESS");
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), label0, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(mvbox), hbox , FALSE, FALSE , 0);
	gtk_container_add(GTK_CONTAINER(wi_frame),mvbox);

	hbox = gtk_hbox_new(TRUE,2);
	label = gtk_label_new("RECEIVED");
	label0 = gtk_label_new(get_transf("rx"));
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), label0, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(mvbox), hbox , FALSE, FALSE , 0);
	gtk_container_add(GTK_CONTAINER(wi_frame),mvbox);

	hbox = gtk_hbox_new(TRUE,2);
	label = gtk_label_new("TRANSMITTED");
	label0 = gtk_label_new(get_transf("tx"));
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), label0, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(mvbox), hbox , FALSE, FALSE , 0);
	gtk_container_add(GTK_CONTAINER(wi_frame),mvbox);

	label1 = gtk_label_new("WIFI STAT");
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), wi_frame, label1);
	
	gtk_container_add(GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), notebook);
	
	gtk_widget_show_all(dialog);

	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(GTK_WIDGET(dialog));
}

void show_about(GtkMenuItem *item, gpointer data)
{ 
	GtkWidget *dialog;
	GtkWidget *label;
	GtkWidget *button_ok;

	dialog = gtk_dialog_new();
	
	gtk_window_set_title(GTK_WINDOW(dialog), ("SMP - About"));
	button_ok = gtk_dialog_add_button(GTK_DIALOG(dialog), "Ok", 0);

	label = gtk_label_new("\nSMP: System Monitor Plugin\n\nThe level bars indicate, in order, the usage percentage of:\ninternal memory(root), mmc card, RAM, CPU\nand WiFi link quality.\n");
	gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox),label);	

	gtk_widget_show_all(dialog);
	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(GTK_WIDGET(dialog));
}

void show_menu(GtkWidget *widget, gpointer data){
	
	SystemMonitorPlugin *smp = (SystemMonitorPlugin *)data;
	

	if (GTK_IS_MENU(smp->menu)) { /* is this a memory leak??? */
	    gtk_widget_destroy(smp->menu_about);
	    gtk_widget_destroy(smp->menu_separator);
	    gtk_widget_destroy(smp->menu_details);
	    gtk_widget_destroy(smp->menu);
    	}
	smp->menu = gtk_menu_new();

	/* Details */
	smp->menu_details = gtk_menu_item_new_with_label("Details");
	gtk_menu_shell_append(GTK_MENU_SHELL(smp->menu), smp->menu_details);
	g_signal_connect(G_OBJECT(smp->menu_details),
			   "activate",
			   G_CALLBACK(show_details),
			   smp);

	/* Separator */
        smp->menu_separator = gtk_separator_menu_item_new();
        gtk_menu_shell_append(GTK_MENU_SHELL(smp->menu), smp->menu_separator);
	/* About */
	smp->menu_about = gtk_menu_item_new_with_label("About");
	gtk_menu_shell_append(GTK_MENU_SHELL(smp->menu), smp->menu_about);
	g_signal_connect(G_OBJECT(smp->menu_about),
			   "activate",
			   G_CALLBACK(show_about),
			   smp);

	gtk_widget_show_all(smp->menu);

	gtk_menu_shell_select_first(GTK_MENU_SHELL(smp->menu), TRUE);

	/* uh uh uh */
	gtk_menu_popup(GTK_MENU(smp->menu), NULL, NULL, pos_menu, data, 1, 0); /*gtk_get_current_event_time());*/

}
/* end  */


void *system_monitor_initialize(HildonStatusBarItem *item, GtkWidget **button)
{
	GtkWidget *image = NULL;
	GError *error = NULL;

	SystemMonitorPlugin *smp = g_new0(SystemMonitorPlugin, 1);

	smp->item=item;
	smp->frame = gtk_hbox_new(FALSE,2);
	smp->icon = gtk_image_new_from_pixbuf(NULL);
	smp->icon_oth = gtk_image_new_from_pixbuf(NULL);
	smp->icon1 = gtk_image_new_from_pixbuf(NULL);
	smp->icon2 = gtk_image_new_from_pixbuf(NULL);
	smp->icon_wi = gtk_image_new_from_pixbuf(NULL);
	*button = smp->button0 = gtk_button_new();
	
	gtk_container_add(GTK_CONTAINER(smp->frame),smp->icon);
	gtk_container_add(GTK_CONTAINER(smp->frame),smp->icon_oth);
	gtk_container_add(GTK_CONTAINER(smp->frame),smp->icon1);
	gtk_container_add(GTK_CONTAINER(smp->frame),smp->icon2);
	gtk_container_add(GTK_CONTAINER(smp->frame),smp->icon_wi);
	gtk_container_add(GTK_CONTAINER(*button),smp->frame);

	/* in buona sostanza button e' il conteiner principale
	 * qui andrebbe data una funzione alla pressione.
	 * */
	
	smp->pixbuf_25 = gdk_pixbuf_new_from_file(LOW_ICON,&error);
	smp->pixbuf_50 = gdk_pixbuf_new_from_file(MEDIUM_ICON,&error);
	smp->pixbuf_75 = gdk_pixbuf_new_from_file(MEDIUM2_ICON,&error);
	smp->pixbuf_full = gdk_pixbuf_new_from_file(FULL_ICON,&error);
	smp->pixbuf_wrong = gdk_pixbuf_new_from_file(WRONG_ICON,&error);
	smp->pixbuf_empty = gdk_pixbuf_new_from_file(EMPTY_ICON,&error);
	smp->pixbuf_curr = gdk_pixbuf_new_from_file(FULL_ICON,&error);
	smp->pixbuf_mem = gdk_pixbuf_new_from_file(MEM_ICON,&error);
	smp->pixbuf_memi = gdk_pixbuf_new_from_file(LOW_ICON,&error);
	smp->pixbuf_disk = gdk_pixbuf_new_from_file(DISK_ICON,&error);
	smp->pixbuf_diski = gdk_pixbuf_new_from_file(LOW_ICON,&error);
	smp->pixbuf_mmc = gdk_pixbuf_new_from_file(MMC_ICON,&error);
	smp->pixbuf_mmci = gdk_pixbuf_new_from_file(LOW_ICON,&error);
	smp->pixbuf_cpu = gdk_pixbuf_new_from_file(CPU_ICON,&error);
	smp->pixbuf_cpui = gdk_pixbuf_new_from_file(LOW_ICON,&error);
	smp->pixbuf_wireless = gdk_pixbuf_new_from_file(WIFI_ICON,&error);
	smp->pixbuf_wifi = gdk_pixbuf_new_from_file(LOW_ICON,&error);

	gdk_pixbuf_composite(smp->pixbuf_cpu,smp->pixbuf_cpui,0,25,16,15,0,25,1,1,GDK_INTERP_NEAREST, 255);
	gdk_pixbuf_composite(smp->pixbuf_mem,smp->pixbuf_memi,0,25,16,15,0,25,1,1,GDK_INTERP_NEAREST, 255);
	gdk_pixbuf_composite(smp->pixbuf_disk,smp->pixbuf_diski,0,20,16,20,0,26,1,1,GDK_INTERP_NEAREST, 255);
	gdk_pixbuf_composite(smp->pixbuf_mmc,smp->pixbuf_mmci,0,20,16,20,0,26,1,1,GDK_INTERP_NEAREST, 255);
	gdk_pixbuf_composite(smp->pixbuf_wireless,smp->pixbuf_wifi,0,25,16,15,0,25,1,1,GDK_INTERP_NEAREST, 255);
	
	gtk_image_set_from_pixbuf(GTK_IMAGE(smp->icon), smp->pixbuf_diski);
	gtk_image_set_from_pixbuf(GTK_IMAGE(smp->icon_oth),smp->pixbuf_mmci);
	gtk_image_set_from_pixbuf(GTK_IMAGE(smp->icon1), smp->pixbuf_memi);
	gtk_image_set_from_pixbuf(GTK_IMAGE(smp->icon2), smp->pixbuf_cpui);
	gtk_image_set_from_pixbuf(GTK_IMAGE(smp->icon_wi), smp->pixbuf_wifi);
 
	/* menu */
	g_signal_connect(G_OBJECT(smp->button0), "clicked", G_CALLBACK(show_menu), smp);
	/* end */

	gtk_widget_show_all(smp->button0);

	smp->timeout_id = gtk_timeout_add(1000, set_pix, smp);

	return (void *)smp;
}

void system_monitor_update(void *data, gint value1, gint value2, const gchar *str)
{
	if ( !data ) {
		return;
	}
	SystemMonitorPlugin *smp = (SystemMonitorPlugin *)data;

	if ( value1 == 0 ) {
		gtk_widget_destroy(GTK_WIDGET(smp->item));
	} else {
		gtk_widget_show_all(smp->button0);
	}

	return;
}

void system_monitor_destroy(void *data)
{
	SystemMonitorPlugin *smp = (SystemMonitorPlugin *)data;
	if (!data) {
		return;
	}

	/* TO FIX  OR  NOT TO FIX */
	g_object_unref(smp->pixbuf_25);
	g_object_unref(smp->pixbuf_50);
	g_object_unref(smp->pixbuf_75);
	g_object_unref(smp->pixbuf_curr);
	g_object_unref(smp->pixbuf_full);
	g_object_unref(smp->pixbuf_wrong);
	g_object_unref(smp->pixbuf_cpu);
	g_object_unref(smp->pixbuf_mem);
	g_object_unref(smp->pixbuf_memi);
	g_object_unref(smp->pixbuf_disk);
	g_object_unref(smp->pixbuf_diski);
	g_object_unref(smp->pixbuf_cpui);
	g_object_unref(smp->pixbuf_wireless);
	g_object_unref(smp->pixbuf_wifi);
	g_object_unref(smp->pixbuf_mmc);
	g_object_unref(smp->pixbuf_mmci);
	g_object_unref(smp->icon);
	g_object_unref(smp->icon_oth);
	g_object_unref(smp->icon_wi);
	g_object_unref(smp->icon1);
	g_object_unref(smp->icon2);
	g_source_remove(smp->timeout_times);
	g_source_remove(smp->timeout_id);
	/* THIS IS THE PROBLEM */

	g_free(data);
}

gint system_monitor_get_priority(void *data)
{
	return 42;
}
